﻿$rootFolderName = $PSScriptRoot
$oldVersion = '38457'
$newVersion = '40938'


Import-Module "${env:ProgramFiles(x86)}\Microsoft Dynamics NAV\80\RoleTailored Client\Microsoft.Dynamics.Nav.Model.Tools.psd1" -force
Import-Module (Join-Path $rootFolderName 'Merge-NAVVersionListString script.ps1') -force


$diffFolderName = (Join-Path $rootFolderName ($oldVersion + 'to' + $newVersion + 'diff'))
$oldObjects = (Join-Path $rootFolderName ($oldVersion + 'objects.txt'))
$newObjects = (Join-Path $rootFolderName ($newVersion + 'objects.txt'))
$newFolder = (Join-Path $rootFolderName ($newVersion + 'update'))
$customObjects = (Join-Path $rootFolderName ($oldVersion + 'customizedobjects.txt'))
$newCustomizedObjects = (Join-Path $rootFolderName ($newVersion  + 'customizedobjects.txt'))
$newCustomizedFolder = (Join-Path $rootFolderName ($newVersion + 'customized'))
if (!(Test-Path $diffFolderName))
{
    mkdir $diffFolderName
}
if (!(Test-Path $newFolder))
{
    mkdir $newFolder
}
if (!(Test-Path $newCustomizedFolder))
{
    mkdir $newCustomizedFolder
}
Write-Host "Comparing customized and original..."
Compare-NAVApplicationObject -Original $oldObjects -Modified $customObjects -Delta $diffFolderName | Where-Object CompareResult -eq 'Identical' | foreach {  Remove-Item (Join-Path $diffFolderName ($_.ObjectType.substring(0,3) + $_.Id + '.delta')) }
Write-Host "Splitting new objects..."
Split-NAVApplicationObjectFile $newObjects $newFolder
Write-Host "Removing unchanged new objects..."
Get-ChildItem -Path $newFolder | foreach { if (!(Test-Path ((Join-Path $diffFolderName $_.BaseName) + '.delta'))) { Remove-Item $_.FullName } }
Write-Host "Updating new objects..."
Update-NAVApplicationObject -Target $newFolder -Delta $diffFolderName -Result $newCustomizedFolder -DateTimeProperty FromModified -ModifiedProperty FromModified -VersionListProperty FromModified -DocumentationConflict ModifiedFirst
Write-Host "Updating customized object version list..."
Get-ChildItem -Path (Join-Path $newCustomizedFolder '*.txt')| foreach { if (Test-Path (Join-Path $newFolder $_.Name)) {Set-NAVApplicationObjectProperty -Target $_.FullName -VersionListProperty (Merge-NAVVersionListString -source (Get-NAVApplicationObjectProperty -Source $_.FullName).VersionList -target (Get-NAVApplicationObjectProperty -Source (Join-Path $newFolder $_.Name)).VersionList) }}
Write-Host "Joining customized object to a single file..."
Join-NAVApplicationObjectFile -Source (Join-Path $newCustomizedFolder '*.txt') -Destination $newCustomizedObjects
Write-Host "If you have conflicts then you need to manually fix conflicting code changes"
